@@ -16,8 +16,6 @@ test_that("console printing", {
 
 test_that("check extention and print document", {
 
-  skip_if_not(has_zip())
-
   x <- read_docx()
   print(x, target = "print.docx")
   expect_true( file.exists("print.docx") )
@@ -53,8 +51,6 @@ test_that("styles_info is returning a tidy df", {
 
 
 test_that("id are sequentially defined", {
-  skip_if_not(has_zip())
-
   doc <- read_docx()
   any_img <- FALSE
   img.file <- file.path( Sys.getenv("R_HOME"), "doc", "html", "logo.jpg" )
@@ -85,12 +81,11 @@ test_that("id are sequentially defined", {
 
 
 test_that("cursor behavior", {
-  skip_if_not(has_zip())
-
   doc <- read_docx() %>%
     body_add_par("paragraph 1", style = "Normal") %>%
     body_add_par("paragraph 2", style = "Normal") %>%
     body_add_par("paragraph 3", style = "Normal") %>%
+    body_bookmark("bkm1") %>%
     body_add_par("paragraph 4", style = "Normal") %>%
     body_add_par("paragraph 5", style = "Normal") %>%
     body_add_par("paragraph 6", style = "Normal") %>%
@@ -109,9 +104,12 @@ test_that("cursor behavior", {
   expect_equal( doc$doc_obj$get_at_cursor() %>% xml_text(), "paragraph 6" )
   doc <- doc %>% cursor_reach(keyword = "paragraph 5")
   expect_equal( doc$doc_obj$get_at_cursor() %>% xml_text(), "paragraph 5" )
+  doc <- doc %>% cursor_bookmark("bkm1")
+  expect_equal( doc$doc_obj$get_at_cursor() %>% xml_text(), "paragraph 3" )
 
 })
 
+
 unlink("*.docx")
 unlink("*.emf")
 
